﻿--[[
Name: LibGuildBankComm-1.0
Revision: $Revision: 13 $
Author(s): Myrroddin of Llane
Description: Saves guild bank contents and sends/receives updates over the addon channel
Dependencies: LibStub, CallbackHandler-1.0, ChatThrottleLib, AceSerializer-3.0
]]

local MAJOR_VERSION = "LibGuildBankComm-1.0"
local MINOR_VERSION = 90000 + tonumber(("$Revision: 13 $"):match("%d+"))

if not LibStub then error(MAJOR_VERSION .. " requires LibStub.") end
local lib, oldversion = LibStub:NewLibrary(MAJOR_VERSION, MINOR_VERSION)
if not lib then return end

local _G = getfenv(0)

local libSerial = LibStub("AceSerializer-3.0", true)
local CBH = LibStub("CallbackHandler-1.0")
local CTL = assert(ChatThrottleLib, MAJOR_VERSION.." requires ChatThrottleLib")

lib.eventFrame = lib.eventFrame or CreateFrame("Frame")
lib.eventFrame:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)
lib.eventFrame:UnregisterAllEvents()

lib.guildBank = lib.guildBank or {}
local ranksCanViewTabs = {} -- store info about guild members' privileges
local playerInfo = {} -- store info about the player's privileges

lib.callbacks = lib.callbacks or CBH:New(lib)

local function commSend(contents, distribution, target)
	CTL:SendAddonMessage("BULK", "gBankComm", contents, distribution, target)
end

function lib.eventFrame:CHAT_MSG_ADDON(prefix, msg, distribution, sender)
    if prefix ~= "gBankComm" then return end
    if sender == "player" then return end
	for cmd, bankData in msg:Deserialize(string.gmatch(("%a+")("%a+"))) do
		if cmd == "REQUEST" then
			local commString = libSerial:Serialize("UPDATE", lib.guildBank[guildName])
			commSend("gBankComm", commString, "WHISPER", sender)
		elseif cmd == "UPDATE" then
			lib.guildBank = bankData
			local guildName = GetGuildInfo("player")
			lib.callbacks:Fire("GuildBankComm_ContentsChanged", sender, bankData, guildName)
		end
	end
end

function lib:RequestDataAboutGuild()
	local numGuildMembers = GetNumGuildMembers()
	local numTabs = GetNumGuildBankTabs()
	for i = 1, numGuildMembers do
		local name, _, rankIndex = GetGuildRosterInfo(i)
		_G.GuildControlSetRank(rankIndex..i)
		for k = 1, numTabs do
			local canView = GetGuildBankTabPermissions(rankIndex..k)
			if canView == 1 then
				ranksCanViewTabs ={
					guildMember = {
						name = UnitName(i),
						rank = rankIndex..i,
						tab = {
							tabNum = numTabs..k,
							canSee = true
						}
					}
				}
			else
				ranksCanViewTabs = {
					guildMember = {
						name = UnitName(i),
						rank = rankIndex..i,
						tab = {
							tabNum = numTabs..k,
							canSee = false
						}
					}
				}
			end
		end
	end
end

function lib:RequestDataAboutPlayer()
	local pName = UnitName("player")
	local guildName, _, rankIndex = GetGuildInfo("player")
	playerInfo.rank = {rank = rankIndex}
	if playerInfo.oldRank ~= playerInfo.rank then
		local numTabs = GetNumGuildBankTabs()
		_G.GuildControlSetRank(rankIndex)
		for i = 1, numTabs do
			local canView = GetGuildBankTabPermissions(numTabs..i)
			if canView == 1 then
				commSend("gBankComm", "REQUEST"..tostring(numTabs..i), "GUILD")
				--[===[@debug@
				print("Player requesting page "..numTabs..i)
				--@end-debug@]===]
			else
				lib.guildBank[i] = nil -- if the player can no longer see a page, nil that page
			end
		end		
		playerInfo.oldRank = playerInfo.rank
	end
	-- the player either can see new pages, or can no longer see pages. in any case, need to update addon
	local bankData = lib.guildBank
	lib.callbacks:Fire("GuildBankComm_ContentsChanged", pName, bankData, guildName)
	--[===[@debug@
	print(pName.." from "..guildName.." has fired GuildBankComm_ContentsChanged")
	--@end-debug@]===]
end

function lib.eventFrame:ACCEPT_GUILD()
	lib:RequestDataAboutPlayer()
	lib:RequestDataAboutGuild()
end

function lib.eventFrame:GUILD_ROSTER_UPDATE()
	lib:RequestDataAboutGuild()
end

function lib.eventFrame:PLAYER_GUILD_UPDATE()
	if not IsInGuild() then -- player left guild			
		local pName = UnitName("player")
		wipe(ranksCanViewTabs)
		wipe(lib.guildBank)
		wipe(playerInfo)
		local bankData = lib.guildBank
		lib.callbacks:Fire("GuildBankComm_PlayerGuildless", pName, bankData)
	else -- something else has changed, like guild name, or player rank
		lib:RequestDataAboutPlayer()
	end
end

function lib:GUILDBANKFRAME_OPENED()	
	lib.guildBank.numTabs = GetNumGuildBankTabs()
	lib.guildBank.guildFunds = GetGuildBankMoney()
	--[===[@debug@
	print("The guild has "..lib.guildBank.guildFunds.." in copper")
	--@end-debug@]===]
	
	local name, icon, view	
	for page = 1, lib.guildBank.numTabs do
		name, icon, view = GetGuildBankTabInfo(page)
		--because I like booleans better just like Starinnia
		if view == 1 then
			view = true
		else
			view = false
		end
		lib.guildBank = {
			[page] = {
				links = {},  --store the slot item links here
				stacks = {}, --store the slot stack counts here
				name = name,
				texture = icon,
				canView = view --or false depending
			}
		}
	end
end

function lib:GUILDBANK_UPDATE_MONEY()
	lib.guildBank.guildFunds = GetGuildBankMoney()
	--[===[@debug@
	local gName = GetGuildInfo("player")
	print(gName.." has "..lib.guildBank.guildFunds.." in copper")
	--@end-debug@]===]
end

function lib:GUILDBANKFRAME_CLOSED()
	local commString = libSerial:Serialize("UPDATE", lib.guildBank)
	local guildie = UnitName(ranksCanViewTabs[name][numTabs].canView)
	if UnitIsConnected(guildie) then
		commSend(string.format("gBankComm", commString), "WHISPER", guildie)
		--@debug
		print("Sending bank info to "..guildie)
		--@end-debug
	end
	local pName = UnitName("player")
	local bankData = lib.guildBank
	local guildName = GetGuildInfo("player")
	lib.CallBacks:Fire("GuildBankComm_ContentsChanged", pName, bankData, guildName)
end

function lib:GUILDBANK_UPDATE_TABS()
	lib.guildBank.numTabs = GetNumGuildBankTabs()
end

local num, itemString
function lib:GUILDBANKBAGSLOTS_CHANGED()
	if (GuildBankFrame and GuildBankFrame:IsVisible()) or (BagnonFrameguildbank and BagnonFrameguildbank:IsVisible()) then
		local page = GetCurrentGuildBankTab()
		--98 slots on a page
		for slot = 1, 98 do
			--see if the slot has an item in it
			if GetGuildBankItemLink(page, slot) then
				--if there is link, parse the ID and place that in table w/ key = slot
				_, _, itemString = string.find(GetGuildBankItemLink(page, slot), "^|c%x+|H(.+)|h%[.+%]")
				lib.guildBank[page].links[slot] = itemString
				
				--get item info too, need for stack sizes in image
				_, num = GetGuildBankItemInfo(page, slot)
				lib.guildBank[page].stacks[slot] = num
			end
			num, itemString = nil, nil
		end
		
		if not lib.guildBank.hasData then
			lib.guildBank.hasData = true
		end
		
		-- time stamp
		lib.guildBank[page].lastScan = time()
	end
end

function lib.eventFrame:PLAYER_ENTERING_WORLD()
	--[===[@debug@
	print("PLAYER_ENTERING_WORLD has fired")
	--@end-debug@]===]
	lib.eventFrame:UnregisterEvent("PLAYER_ENTERING_WORLD")
end

function lib:Start()	
	lib.eventFrame:RegisterEvent("CHAT_MSG_ADDON")
	if IsInGuild() then
		lib.eventFrame:RegisterEvent("PLAYER_GUILD_UPDATE")
		lib.eventFrame:RegisterEvent("GUILD_ROSTER_UPDATE")
		lib.eventFrame:RegisterEvent("GUILDBANKBAGSLOTS_CHANGED")
		lib.eventFrame:RegisterEvent("GUILDBANKFRAME_OPENED")
		lib.eventFrame:RegisterEvent("GUILDBANKFRAME_CLOSED")
		lib.eventFrame:RegisterEvent("GUILDBANK_UPDATE_TABS")
		lib.eventFrame:RegisterEvent("GUILDBANK_UPDATE_MONEY")
		
		_G.GuildRoster(); -- force GUILD_ROSTER_UPDATE() which is throttled to 1 every 10 seconds		
		lib:RequestDataAboutPlayer()
		lib:RequestDataAboutGuild()		
		local  _, _, rankIndex = GetGuildInfo("player")		
		playerInfo.oldRank = {oldRank = rankIndex}
	else		
		lib.eventFrame:RegisterEvent("ACCEPT_GUILD")
	end
	lib.eventFrame:RegisterEvent("PLAYER_ENTERING_WORLD")	
end
lib:Start()